/*
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013 Alexis Megas.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from Dooble without specific prior written permission.
**
** DOOBLE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
** OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
** IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
** INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
** NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
** DOOBLE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _dexceptionswindow_h_
#define _dexceptionswindow_h_

#include <QMainWindow>

#include "dexceptionsmodel.h"
#include "ui_exceptionsWindow.h"

class QCloseEvent;
class QProgressBar;

class dexceptionswindow: public QMainWindow
{
  Q_OBJECT

 public:
  dexceptionswindow(dexceptionsmodel *model);
  ~dexceptionswindow();
  bool allowed(const QString &host);
  bool contains(const QString &host) const;
  void clear(void);
  void populate(void);
  void reencode(QProgressBar *progress);
  void closeEvent(QCloseEvent *event);
  qint64 size(void) const;
  QStringList allowedHosts(void) const;

 private:
  Ui_exceptionsWindow ui;
  bool event(QEvent *event);
  void saveState(void);
  void keyPressEvent(QKeyEvent *event);

 public slots:
  void slotShow(void);
  void slotSetIcons(void);

 private slots:
  void slotAdd(const QString &host,
	       const QUrl &url,
	       const QDateTime &dateTime);
  void slotSort(int column, Qt::SortOrder order);
  void slotAllow(void);
  void slotClose(void);
  void slotDelete(void);
  void slotDeleteAll(void);
  void slotTextChanged(const QString &text);
  void slotItemsSelected(const QItemSelection &selected,
			 const QItemSelection &deselected);

 signals:
  void iconsChanged(void);
};

#endif
